/* Extended logging daemon for the OS/2 Forum Austria Extended      */
/* GoServe Filter.                                                  */
/* This program will be started by the OS/2 Forum Austria Extended  */
/* GoServe Filter and will terminate after some idle time.          */
/* ================================================================ */
/* Written by by Michael Warmuth for OS/2 Forum Austria.            */
/* (E-mail: Michael.Warmuth@wu-wien.ac.at).                         */
/* For using and copiing see the file copyright.doc.                */

/* Set global vars */
reqq = 'OFAEGFLOGREQ'
cmdq = 'OFAEGFLOGCMD'
synq = 'OFAEGFLOGSYN'
f.badchars = '/\'xrange('00'x,'20'x)xrange('21'x,'2f'x)xrange('3a'x,'40'x)xrange('5b'x,'5e'x)'60'x||,
      xrange('7b'x,'ff'x)
f.goodchars = '!!'copies('?',length(f.badchars)-2)
f.cline.0 = 0

/* Open contol file */
extlogcf = 'extlogd.rcf'
do while stream(extlogcf,'c','open')\='READY:'; call SysSleep 1; end
call rxqueue 'delete', synq

oldq = rxqueue('get')                        /* Get original queuename */

/* Reset clock */
idlestart = time('r')

extlogcnt = 0
f.lastwrite = 0
do until time('e')-idlestart>60
   /* Init sleep flag */
   dosleep = 0

   /* Process all commands */
   call rxqueue 'set', cmdq
   do while queued()>0
      parse pull cmdcmd ansq f.loggi_count f.sel
      call rxqueue 'set', ansq
      queue cmdcount(cmdcmd)
      call rxqueue 'set', cmdq
      idlestart = time('e')
   end /* do */

   /* Process one logging request */
   call rxqueue 'set', reqq
   if queued()>0 then do
      parse pull reqcmd reqparms
      select
         when reqcmd='REQUEST' then do
            parse var reqparms f.loggi_reque f.date f.time f.rsel f.sel f.file
            call logrequ
         end  /* Do */
         when reqcmd='REDIRECT' then do
            parse var reqparms f.loggi_redir f.date f.time f.sel f.redirurl
            call logredir
         end  /* Do */
         when reqcmd='COUNT' then do
            parse var reqparms f.loggi_count f.date f.time f.sel
            call logcount
            extlogcnt = extlogcnt + 1
         end  /* Do */
         otherwise nop
      end  /* select */
      idlestart = time('e')
   end  /* Do */
   else dosleep = 1


   /* Write out counter log file */
   call rxqueue 'set', cmdq
   if queued()=0 & (extlogcnt>100 | (extlogcnt>0 & time('e')-idlestart>10)) then do
      extlogcnt = writecount(extlogcnt)
   end  /* Do */

   /* When idle then sleep for one second */
   if dosleep then do
      call SysSleep 1
   end  /* Do */
end /* do */

/* Close control file */
call stream extlogcf, 'c', 'close'

exit


/* ----------------------------------------------------------------------- */
/* CMDCOUNT: Answer a counter command request.                             */
/* ----------------------------------------------------------------------- */
cmdcount: procedure expose f.

/* Get parameter */
parse arg c.cmd

/* Look if line exists */
i = get_line()

/* Get line */
parse var f.cline.i '['c.retdate c.rettime']' . c.retnum .

select
   /* Date */
   when c.cmd='FLASTACCDATE' then do
      parse var c.retdate year '/' month '/' day
      retc = month'/'day'/'year
   end  /* Do */
   /* European date */
   when c.cmd='FLASTACCEDATE' then do
      parse var c.retdate year '/' month '/' day
      retc = day'.'month'.'year
   end  /* Do */
   /* Time */
   when c.cmd='FLASTACCTIME' then do
      retc = c.rettime
   end  /* Do */
   /* Number */
   when c.cmd='FACCNUMBER' then do
      retc = c.retnum + 1
   end  /* Do */
   otherwise
      retc = ''
end  /* Select */

return retc


/* ----------------------------------------------------------------------- */
/* LOGREQU: Do request logging.                                            */
/* ----------------------------------------------------------------------- */
logrequ: procedure expose f.

/* Create log file if not existing */
if stream(f.loggi_reque,'c','query exists')='' then do
   call lineout f.loggi_reque, '[  date     time  ] request     corrected request     file sent (if any)'
   call lineout f.loggi_reque, copies('-',79)
   call stream f.loggi_reque, 'c', 'close'
end  /* Do */

/* Open log file */
do while stream(f.loggi_reque,'c','open')\='READY:'; call SysSleep 1; end
call stream f.loggi_reque, 'c', 'seek <0'

/* Do the logging */
call lineout f.loggi_reque, '['f.date f.time']' f.rsel f.sel f.file

/* Close log file */
call stream f.loggi_reque, 'c', 'close'

return


/* ----------------------------------------------------------------------- */
/* LOGREDIR: Do redirection logging.                                       */
/* ----------------------------------------------------------------------- */
logredir: procedure expose f.

/* Create log file if not existing */
if stream(f.loggi_redir,'c','query exists')='' then do
   call lineout f.loggi_redir, '[  date     time  ] request     referer'
   call lineout f.loggi_redir, copies('-',79)
   call stream f.loggi_redir, 'c', 'close'
end  /* Do */

/* Open log file */
do while stream(f.loggi_redir,'c','open')\='READY:'; call SysSleep 1; end
call stream f.loggi_redir, 'c', 'seek <0'

/* Do the logging */
call lineout f.loggi_redir, '['f.date f.time']' f.sel f.redir

/* Close log file */
call stream f.loggi_redir, 'c', 'close'

return


/* ----------------------------------------------------------------------- */
/* LOGCOUNT: Do counter logging.                                           */
/* ----------------------------------------------------------------------- */
logcount: procedure expose f.

/* Look if line exists */
i = get_line()

/* Get line */
c.lineno = i
parse var f.cline.i . ']' c.nextrequ c.nextnum c.orignum
c.line = c.nextrequ c.nextnum+1 c.orignum

f.cline.i = '['f.date f.time']' c.line

/* Drop used vars */
drop c.

return


/* ----------------------------------------------------------------------- */
/* WRITECOUNT: Write out all counter logging lines.                        */
/* ----------------------------------------------------------------------- */
writecount: procedure expose f.

/* Get argument */
parse arg extlogcnt

/* Create log file if not existing */
if stream(f.loggi_count,'c','query exists')='' then do
   call lineout f.loggi_count, '[  date     time  ] request     number'
   call lineout f.loggi_count, copies('-',79)
   call stream f.loggi_count, 'c', 'close'
end  /* Do */

/* Open log file */
do while stream(f.loggi_count,'c','open')\='READY:'; call SysSleep 1; end

/* Backup log file */
address CMD '@type' f.loggi_count '1>' left(f.loggi_count,pos('.',f.loggi_count))'bak 2> nul'

/* Get file content */
c.logcont = charin(f.loggi_count,1,chars(f.loggi_count))

/* Get header lines */
c.loghead = ''
do 2
   parse var c.logcont c.lines '0d0a'x c.logcont
   c.loghead = c.loghead || c.lines || '0d0a'x
end  /* Do */

/* Log all lines */
do i=f.lastwrite+1 to f.cline.0 until queued()>0
   parse var f.cline.i '[' gotdate gottime ']' f.sel gotnum orignum
   if gotnum-orignum>0 then do
      f.cline.i = write_logging(gotdate,gottime,gotnum-orignum)
   end  /* Do */
   nowwrite = i
end /* do */
do i=1 to f.lastwrite while queued()=0
   parse var f.cline.i '[' gotdate gottime ']' f.sel gotnum orignum
   if gotnum-orignum>0 then do
      f.cline.i = write_logging(gotdate,gottime,gotnum-orignum)
   end  /* Do */
   nowwrite = i
end /* do */
if nowwrite>=f.cline.0 then f.lastwrite = 0
else f.lastwrite = nowwrite

/* If all changes have been written reset counter */
if queued()=0 then extlogcnt = 0

/* Do the logging */
call stream f.loggi_count, 'c', 'seek =1'
call charout f.loggi_count, c.loghead || c.logcont

/* Close log file */
call stream f.loggi_count, 'c', 'close'


return extlogcnt


/* ----------------------------------------------------------------------- */
/* WRITE_LOGGING: Write out one counter logging line.                      */
/* ----------------------------------------------------------------------- */
write_logging: procedure expose f. c.logcont

/* Get parameters */
parse arg gotdate, gottime, gotnum

/* Search line */
c.tlogcont = translate(c.logcont)
c.foundword = wordpos(translate(f.sel),c.tlogcont)
if c.foundword\=0 then do
   c.foundpos = wordindex(c.tlogcont,c.foundword)
   c.nextrequ = word(c.logcont,c.foundword)
   c.tmpword = word(c.logcont,c.foundword+1)
   parse var c.tmpword c.nextnum '0d0a'x c.tmpword
   c.nextpos = wordindex(c.tlogcont,c.foundword+2)

   gotnum = c.nextnum+gotnum
   retc = '['gotdate gottime']' c.nextrequ gotnum
   if c.nextpos=0 then do
      c.logcont = left(c.logcont,c.foundpos-21) || retc || '0d0a'x || c.tmpword
   end  /* Do */
   else do
      c.logcont = left(c.logcont,c.foundpos-21) || retc || '0d0a'x || c.tmpword,
            substr(c.logcont,c.nextpos)
   end  /* Do */
   retc = retc gotnum
end  /* Do */
else do
   c.logleft = ''
   c.found = 0
   c.eof = 0
   do while \c.found & \c.eof
      /* Get next line */
      parse var c.logcont c.lines '0d0a'x c.logcont
      parse var c.lines '[' c.nextdate c.nexttime ']' c.nextrequ c.nextnum
   
      /* Set eof flag if at end of file */
      if pos('1a'x,c.lines)\=0 | pos('0d0a'x,c.logcont)=0 then do
            c.eof = 1
      end  /* Do */
   
      /* Line not found */
      if translate(c.nextrequ)>translate(f.sel) | c.eof then do
         c.found = 1
         retc = '['gotdate gottime']' f.sel gotnum
         c.logleft = c.logleft || retc || '0d0a'x || c.lines || '0d0a'x
         retc = retc gotnum
      end  /* Do */
      else c.logleft = c.logleft || c.lines || '0d0a'x
   end  /* Do */
   c.logcont = c.logleft || c.logcont
end  /* Do */


/*c.logleft = ''
c.found = 0
c.eof = 0
do while \c.eof & \c.found
   /* Get next line */
   parse var c.logcont c.lines '0d0a'x c.logcont
   parse var c.lines '[' c.nextdate c.nexttime ']' c.nextrequ c.nextnum

   /* Set eof flag if at end of file */
   if pos('1a'x,c.lines)\=0 | pos('0d0a'x,c.logcont)=0 then do
         c.eof = 1
   end  /* Do */

   /* If not found already */
   if \c.found then do
      select
         /* Found the line */
         when translate(c.nextrequ)=translate(f.sel) then do
            c.found = 1
            gotnum = c.nextnum+gotnum
            retc = '['gotdate gottime']' c.nextrequ gotnum
            c.logleft = c.logleft || retc || '0d0a'x
            retc = retc gotnum
         end  /* Do */
         /* Line not found */
         when translate(c.nextrequ)>translate(f.sel) | c.eof then do
            c.found = 1
            retc = '['gotdate gottime']' f.sel gotnum
            c.logleft = c.logleft || retc || '0d0a'x || c.lines || '0d0a'x
            retc = retc gotnum
         end  /* Do */
         otherwise c.logleft = c.logleft || c.lines || '0d0a'x
      end  /* Select */
   end  /* Do */
end  /* Do */
c.logcont = c.logleft || c.logcont*/

return retc


/* ----------------------------------------------------------------------- */
/* GET_LINE: Get one line of the counter logging file.                     */
/* ----------------------------------------------------------------------- */
get_line: procedure expose f.

interpret "c.found = (symbol('f.cind."translate(f.sel,f.goodchars,f.badchars)"')\='LIT')"
if c.found then do
   interpret 'retc=f.cind.'translate(f.sel,f.goodchars,f.badchars)
end  /* Do */
else do
   f.cline.0 = f.cline.0 + 1
   retc = f.cline.0
   interpret 'f.cind.'translate(f.sel,f.goodchars,f.badchars)'=retc'

   if stream(f.loggi_count,'c','query exists')\='' then do
      do while SysFileSearch(' 'f.sel' ',f.loggi_count,'c.line.')=3; call SysSleep 1; end
   end  /* Do */
   else do
      c.line.0 = 0
   end  /* Do */

   if c.line.0=0 then do
      f.cline.retc = '[00/00/00 00:00:00]' f.sel 0 0
   end  /* Do */
   else do
      parse var c.line.1 . ']' . c.oldnum
      f.cline.retc = c.line.1 c.oldnum
   end  /* Do */
end  /* Do */

return retc
